//---------------------------------------------------------------------------
#ifndef MixmasterH
#define MixmasterH

#include <vcl/comctrls.hpp>
//---------------------------------------------------------------------------
bool __fastcall HasMix();
bool __fastcall HasMixmaster();

AnsiString __fastcall MixDir();
AnsiString __fastcall MixPublicRing();

// constants used for help windows
const AnsiString MixmasterDocs = "Mixmaster Online Documents"; // caption
const AnsiString UpdateMixDocs = "UpdateMixDocs"; // help section
const AnsiString MixKeyDuplicate = "Duplicate Mixmaster Keys";
const AnsiString MixKeyNotFound = "Mixmaster Keys Not Found";
const AnsiString MixKeyNew = "New Mixmaster Keys";
const AnsiString MixUnknownRemailer = "<unknown remailer, no key/description available>";

AnsiString __fastcall MixGetNewRemailerKeys(TStrings *KeyDoc);
void __fastcall MixAddNewRemailerKeys(TStrings *KeyDoc, TStrings *Type2);
AnsiString __fastcall MixGenType2(TStrings *Type2, TStrings *Mring);
AnsiString __fastcall MixListKeyNotFound(TStrings *Mlist, TStrings *Type2);
AnsiString __fastcall MixType2Doubles(TStrings *Type2);

bool __fastcall MixRingLineOk(AnsiString &line);
bool __fastcall MixListLineOk(AnsiString &line);

const AnsiString MixKey = "Software\\Mixmaster";
const AnsiString MixExe = "mix.exe";
const AnsiString MixTmp = "mix.tmp.QS";
const AnsiString MixCfg = "mix.cfg";
const AnsiString MixOut = "out*.txt";
const AnsiString DefMixCfg =
   "###### Client Config ######\r\nSENDMAIL outfile\r\nERRLOG errlog\r\nREMAIL n";

#define QUEUEDIRECTORY  String(MixDir()+"pool\\")
#define QUEUEFILES  String(QUEUEDIRECTORY+MixOut)

#define PATHMAX 512
#define LINELEN 128

class TMixKey : public TObject
{
private:
   TStringList *key;
   AnsiString keyring;
   AnsiString GetKeyring();
   AnsiString GetName();
   AnsiString GetUserId();
   AnsiString GetIdText32();
   AnsiString GetVersion();
   AnsiString GetCaps();
   AnsiString GetDescription();
   //int GetType();
   //long GetTimeStamp();
   //int GetValidity();
   //byte GetAlgorithm();
   //int GetKeyBits();
   //int GetKeyUserId();
   //int GetMPI(BUFFER *mpi);
   //int PutMPI(BUFFER *buf, BUFFER *mpi);
   ///int KeyId(BUFFER *Keyid);
   //bool GetIsPair();
   //Char *GetKey();
public:
   TMixKey *Next;
   TMixKey *Prev;
   virtual __fastcall TMixKey(TStringList *Key, AnsiString Keyring="");
   virtual __fastcall TMixKey(TMixKey *Key);
   __fastcall ~TMixKey();
   __property AnsiString Description = { read=GetDescription };
   __property AnsiString Name = { read=GetName };
   __property AnsiString UserId = { read=GetUserId };
   __property AnsiString IdText32 = { read=GetIdText32 };
   __property AnsiString Version = { read=GetVersion };
   __property AnsiString Caps = { read=GetCaps };
   __property AnsiString Keyring = { read=GetKeyring };
   __property TStringList *Key = { read=key };
   long __fastcall Write(TFileStream *Stream);
   //__property int Type = { read=GetType };
   //__property long TimeStamp = { read=GetTimeStamp };
   //__property int Validity = { read=GetValidity };
   //__property byte Algorithm = { read=GetAlgorithm };
   //__property int KeyBits = { read=GetKeyBits };
   //__property bool IsPair = { read=GetIsPair };
};

// db errorcodes
const MixDBKeyringNotFound = -1;

class TMixDb : public TObject
{
   TMixKey *lastkey;
   TMixKey *firstkey;
   AnsiString keyring;
   int GetCount();
   bool modified;
public:
   virtual __fastcall TMixDb(AnsiString PubRing);
   virtual __fastcall TMixDb(TStrings *PubKeys);
   virtual __fastcall ~TMixDb();
   void __fastcall SaveToFile(AnsiString &FileName);
   void __fastcall SaveToStrings(TStrings *strings);
   AnsiString __fastcall SaveToString();
   __property int KeyCount = { read=GetCount };
   __property bool Modified = { read=modified };
   TMixKey* NextKey(TMixKey *FromThis);
   TMixKey* PrevKey(TMixKey *FromThis);
   TMixKey* KeyWithIdText32(AnsiString Text32);
   TMixKey* GetKey(int pos);
   TMixKey* FirstKey();
   TMixKey* LastKey();
   int __fastcall AddKey(TMixKey* Key);
   int __fastcall Find(TMixKey* Key);
   int __fastcall FindDescrip(AnsiString &Descrip);
   int __fastcall Delete(int pos);
   void __fastcall Clear();
   void __fastcall OrderBy(TStrings *type2);
};

typedef unsigned char byte;
typedef struct {
  byte *data;
  long length;
  long ptr;
  long size;
  byte sensitive;
} BUFFER;

typedef struct {
  char *name;
  FILE *f;
} LOCK;

typedef struct {
  int filetype;
  BUFFER *db;
  LOCK *lock;
  int modified;
  char filename[LINELEN];
  BUFFER *encryptkey;
} KEYRING;

int mixfile(char *path, const char *name);
FILE *mix_openfile(const char *name, const char *a);

int __fastcall mix_encrypt(int type, BUFFER *message, char *chain, int numcopies,
 	 BUFFER *feedback);
int __fastcall mix_encrypt_exe(int type, BUFFER *message, char *chain, int numcopies,
   BUFFER *feedback, bool *cancel, TProgressBar *progressbar);
//int PgpIsConventional(BUFFER *b);

int buf_readfile(BUFFER *buffer, char *filename);
int buf_writefile(BUFFER *buffer, char *filename, char* mode);  // mode = 'wb' or 'wt'
int __fastcall buf_read(BUFFER *outmsg, FILE *infile);
int __fastcall buf_write(BUFFER *buffer, FILE *out);

BUFFER* __fastcall buf_new(void);
int __fastcall buf_free(BUFFER *buf);
int __fastcall buf_reset(BUFFER *buf);
int __fastcall buf_append(BUFFER *buffer, byte *mess, int len);
int __fastcall buf_sets(BUFFER *buffer, char *s);
void __fastcall buf_move(BUFFER *dest, BUFFER *src);
int __fastcall buf_write(BUFFER *buffer, FILE *out);
int __fastcall mix_send(void);
int __fastcall pgp_encrypt(int mode, BUFFER *message, BUFFER *encr, BUFFER *sigid,
		BUFFER *pass, char *pubring, char *secring);
int __fastcall pgp_mailenc(int mode, BUFFER *message, char *sigid,
		BUFFER *pass, char *pubring, char *secring);
int __fastcall pgp_decrypt(BUFFER *message, BUFFER *pass, BUFFER *sig, char *pubring,
		char *secring);

int __fastcall rnd_add(byte *b, long len);
void __fastcall rnd_time();
int __fastcall rnd_mouse(UINT i, WPARAM w, LPARAM l);
int __fastcall buf_getline(BUFFER *buffer, BUFFER *line);
int __fastcall buf_getheader(BUFFER *buffer, BUFFER *field, BUFFER *content);
int __fastcall buf_prepare(BUFFER *buffer, int size);
int __fastcall buf_appendc(BUFFER *to, byte b);
int __fastcall buf_appendi(BUFFER *to, int i);
int __fastcall buf_appendl(BUFFER *to, long l);
int __fastcall buf_cat(BUFFER *to, BUFFER *from);
int __fastcall buf_clear(BUFFER *buffer);
int __fastcall buf_appends(BUFFER *buffer, char *s);
int __fastcall buf_get(BUFFER *buffer, BUFFER *to, int n);
int __fastcall buf_rest(BUFFER *to, BUFFER *from);
int __fastcall buf_nl(BUFFER *b);
int __fastcall buf_set(BUFFER *to, BUFFER *from);
int __fastcall decode(BUFFER *in, BUFFER *out);
int __fastcall encode(BUFFER *in, int linelen);
int __fastcall buf_getc(BUFFER *buffer); // copy
int __fastcall buf_geti(BUFFER *b); // copy
long __fastcall buf_getl(BUFFER *b); //copy
int __fastcall buf_rewind(BUFFER *buffer); //copy
int __fastcall buf_eq(BUFFER *b1, BUFFER *b2); // copy
int __fastcall strifind(const char *string, const char *keyword);

// internal use for accessing dll function with run-time binding
typedef int (*BUF_READ)(BUFFER*, FILE*);
typedef int (*BUF_WRITE)(BUFFER*, FILE*);
typedef BUFFER* (*BUF_NEW)(void);
typedef int (*BUF_FREE)(BUFFER*);
typedef int (*BUF_RESET)(BUFFER*);
typedef int (*BUF_APPEND)(BUFFER*, byte*, int);
typedef int (*BUF_SETS)(BUFFER*, char *);
typedef void(*BUF_MOVE)(BUFFER*, BUFFER*);
typedef int (*BUF_WRITE)(BUFFER*, FILE*);
typedef int (*MIX_ENCRYPT)(int, BUFFER*, char*, int, BUFFER*);
typedef int (*MIX_SEND)(void);
typedef int (*PGP_ENCRYPT_BUF)(int, BUFFER*, BUFFER*, BUFFER*, BUFFER*, char*, char*);
typedef int (*PGP_MAILENC_BUF)(int, BUFFER*, char*, BUFFER*, char*, char*);

typedef int (*PGP_DECRYPT)(BUFFER*, BUFFER*, BUFFER*, char*, char*);

typedef int (*RND_ADD)(byte*,long);

typedef void (*RND_TIME)(void);

typedef int (*RND_MOUSE)(UINT, WPARAM, LPARAM);

typedef int (*BUF_GETLINE)(BUFFER*, BUFFER*);
typedef int (*BUF_GETHEADER)(BUFFER*, BUFFER*, BUFFER*);
typedef int (*BUF_PREPARE)(BUFFER*, int);
typedef int (*BUF_APPENDC)(BUFFER*, byte);
typedef int (*BUF_APPENDL)(BUFFER*, long);
typedef int (*BUF_APPENDI)(BUFFER*, int);
typedef int (*BUF_CAT)(BUFFER*, BUFFER*);
typedef int (*BUF_CLEAR)(BUFFER*);
typedef int (*BUF_APPENDS)(BUFFER*, char*);
typedef int (*BUF_GET)(BUFFER*, BUFFER*, int);
typedef int (*BUF_REST)(BUFFER*, BUFFER*);
typedef int (*BUF_NL)(BUFFER*);
typedef int (*BUF_SET)(BUFFER*, BUFFER*);
typedef int (*DECODE)(BUFFER*, BUFFER*);
typedef int (*ENCODE)(BUFFER*, int);
typedef int (*STRIFIND)(const char*, const char*);
// packet type
#define PGP_ENCRYPTED 9

#define MSG_MAIL 1
#define PGP_PASS -2
#define PGP_ERR -1
#define PGP_SIGN 1
#define PGP_ENCRYPT 2
#define PGP_TEXT 16
#define PGP_REMAIL 8
#define PGP_CONVENTIONAL 4
#define PGP_NOARMOR 32

// error codes
#define PGP_OK 0	 	    // valid message, not signed
#define PGP_SIGOK 1	    // valid signature
#define PGP_NOMSG 2		 // is not a OpenPGP message
#define PGP_NODATA 3	 // OpenPGP packet does not contain user data
#define PGP_SIGNKEY 4	 // can't verify signature
#define PGP_ERR -1		 // can't read message, no matching key found
#define PGP_PASS -2		 // bad passphrase
#define PGP_SIGBAD -3	 // bad signature

#define BUFFER_MAX 2*1024*1024
#define BUFSIZE 4096
#define MixRandBin MixDir()+"mixrand.bin"
#define MixRandPool AnsiString("RandomPool")
//---------------------------------------------------------------------------
extern HINSTANCE hMixLib;
extern HINSTANCE hSSLLib;
//---------------------------------------------------------------------------
#endif
